/** 
 *   Tipper - Tolar: Power dc/dc unit fort wireles equipment.
 *
 *   (c) Copyright 2006 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   14/07/2006: Created.
 **/

/**
 * At this moment the switch port is implemented which ioperm(); 
 **/

#define _GNU_SOURCE

#define SWITCH_PORT 0x378

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/io.h>
#include "my_stdio.h"
#include "version.h"

# define SWITCH_1  0x01

static int io_perm = 0;
static int as_port;
static int verbose_flag = 0;

static void my_outb(unsigned char value, unsigned short int port)
{
	PVERB(PINFO,"# outb 0x%02x, 0x%03x\n", (unsigned char) value, port);
#ifndef DEMO_MODE
	outb(value, port);
#endif
	return;
}


int switchs_open(int port)
{
	as_port = port;
#ifdef DEMO_MODE
	io_perm = 1;
	return 0;
#endif
	/* Get access to the ports */
	if (as_port < 0x3ff ) {	
		if (ioperm(as_port, 1, 1)) {
			perror("ioperm");
			return -1;
		}
	}else{
		if (iopl(3) != 0) {
			perror("iopl");
			return -1;
		}
	}
	io_perm = 1;

	return 0;
}


int switchs_close(void)
{
#ifndef DEMO_MODE
	/* Release access to the ports */
	if (ioperm(as_port, 1, 0)) {
		perror("ioperm"); 
		return -1;
	}
#endif
	io_perm = 0;

	return 0;
}


int switchs_set(int switchs)
{
	
	if (!io_perm) return -1;

	switch (switchs) {
	case 0: my_outb(0, as_port); break;
	case 1: my_outb(SWITCH_1, as_port); break;
	default:
		return -1;
	}

	return 0;
}



int main(int argc, char * const argv[])
{
        int op;
        int switch_port = SWITCH_PORT;
	int value = -1;
	SET_VERBOSE_FLAG(PERR);

#ifdef DEMO_MODE
	fprintf(stderr,"# ***\n# Warning: Compiled with DEMO_MODE\n# ***\n");
#endif

        while  ((op = getopt(argc, argv, "vp:nfh")) != EOF)
                switch( op ){

                case 'v':
			INCREMENT_VERBOSE_FLAG();
                        break;
                case 'p':
                        switch_port = strtol(optarg, (char **)NULL, 16);
                        break;
		case 'n':
			value = 1;
			break;
		case 'f':
			value = 0;
			break;
                case 'h':
                default:
                        printf("Use %s [-v (incremental)] [-p port] -n|-f\n",
                               argv[0]);
                        exit(-1);
                }
	if (value != -1) {
		if (switchs_open(switch_port) != 0) {
			PVERB(PERR, "Cannot acces switch port.\n");
			exit(1);
		}
		switchs_set(value);
		switchs_close();
	} else {
		printf("Missing argument: -n|-f\n");
	}
        return 0;
}
